/*
	Ritma - Advanced metronome for GBA
	Copyright (C) 2005  Avelino Herrera Morales

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "gba.h"
#include "font_8x8.h"

#define  PRESSED(k)  (!(REG_KEYPAD & (k)))

#define  RANDOM_0_100  (rand() % 101)

int __gba_multiboot;

/* modo grfico */
#define BG0_ENABLE      0x100
#define BG1_ENABLE      0x200
#define BG2_ENABLE      0x400
#define BG3_ENABLE      0x800
#define OBJ_ENABLE      0x1000
#define WIN0_ENABLE     0x2000
#define WIN1_ENABLE     0x4000
#define WINOBJ_ENABLE   0x8000

#define BACK_BUFFER     0x10

#define MODE0           0x0
#define MODE1           0x1
#define MODE2           0x2
#define MODE3           0x3
#define MODE4           0x4
#define MODE5           0x5

#define  VIDEO_BUFFER_0  ((u16 *) 0x6000000)
#define  VIDEO_BUFFER_1  ((u16 *) 0x600A000)
#define  BG_PALETTE_MEM  ((u16 *) 0x5000000)

#define  TOP_LEFT      0
#define  TOP_RIGHT     239
#define  BOTTOM_LEFT   38060
#define  BOTTOM_RIGHT  38299

#define RGB16(r,g,b)  ((r) + ((g) << 5) + ((b) << 10))

#define  ERASE_SCREEN(scr)  memset(scr, 0, 240 * 160)

#define  MAX_BPM  300
#define  MIN_BPM  10

#define  MAX_BEAT  16
#define  MIN_BEAT  1

u16 *current_video_buffer = VIDEO_BUFFER_0;

#define  GET_N(f)     (2048 - (131072 / (f)))
#define  PLAY_TICK_A  {REG_SOUND1CNT_L = 0x0000; REG_SOUND1CNT_H = 60 | SOUNDDUTY50 | SOUND1ENVINIT(15); REG_SOUND1CNT_X = GET_N(2000) | SOUND1PLAYONCE | SOUNDINIT;}
#define  PLAY_TICK_B  {REG_SOUND1CNT_L = 0x0000; REG_SOUND1CNT_H = 60 | SOUNDDUTY50 | SOUND1ENVINIT(15); REG_SOUND1CNT_X = GET_N(1000) | SOUND1PLAYONCE | SOUNDINIT;}

#define  STATUS_SO_CLEAR_SC_CLEAR  0
#define  STATUS_SO_SET_SC_CLEAR    1
#define  STATUS_SO_CLEAR_SC_SET    2
#define  STATUS_SO_SET_SC_SET      3

u16 status = STATUS_SO_CLEAR_SC_CLEAR;


void InterruptProcess(void) __attribute__ ((section(".iwram")));   // the ISR must be declared in this way

 
void InterruptProcess(void) {
	// clear interrupt flag
	REG_IF |= REG_IF;
}


void flip_and_show(void) {
	if (current_video_buffer == VIDEO_BUFFER_1) {
		REG_DISPCNT |= BACK_BUFFER;
		current_video_buffer = VIDEO_BUFFER_0;
	}
	else {
		REG_DISPCNT &= ~BACK_BUFFER;
		current_video_buffer = VIDEO_BUFFER_1;
	}
}


void put_character(char c, int x, int y, u16 *video_buffer, int erase) {
	int font_y = (((int) c) - 32) / 16;
	int font_x = (((int) c) - 32) % 16;
	int i, j;
	int font_offset = (font_y * 64 * 8) + (font_x * 4);
	int offset = ((y * 240) + x) >> 1;
	for (i = 0; i < 8; i++, offset += 120, font_offset += 64) {
		if (erase) {
			for (j = 0; j < 4; j++)
				video_buffer[offset + j] = 0x0000;
		}
		for (j = 0; j < 4; j++) {
			register u16 d = fontData[font_offset + j];
			register u16 prev = video_buffer[offset + j];
			register u16 to_write = 0;
			if ((d & 0x00FF) != fontTransparentIndex)
				to_write |= (d & 0x00FF);
			else
				to_write |= (prev & 0x00FF);
			if ((d >> 8) != fontTransparentIndex)
				to_write |= (d & 0xFF00);
			else
				to_write |= (prev & 0xFF00);
			video_buffer[offset + j] = to_write;
		}
	}
}


void put_string(char *s, int x, int y, u16 *video_buffer, int erase) {
	int i = 0;
	while (s[i] != 0) {
		put_character(s[i], x, y, video_buffer, erase);
		x += 8;
		i++;
	}
}


void wait_for_vsync() {
	while (((volatile u16) REG_VCOUNT) != 160)
		;
}


#define  GET_X_CENTER(width)  (120 - ((width) * 4))

void show_main_screen() {
	ERASE_SCREEN(current_video_buffer);
	put_string("External Connector Test", GET_X_CENTER(23), 30, current_video_buffer, 0);
	put_string(" Start: SO=0, SC=0", GET_X_CENTER(18), 50, current_video_buffer, 0);
	put_string("     A: SO=1, SC=0", GET_X_CENTER(18), 60, current_video_buffer, 0);
	put_string("     B: SO=0, SC=1", GET_X_CENTER(18), 70, current_video_buffer, 0);
	put_string("Select: SO=1, SC=1", GET_X_CENTER(18), 80, current_video_buffer, 0);
	if (status == STATUS_SO_CLEAR_SC_CLEAR)
		put_string("Status: SO=0  SC=0", GET_X_CENTER(18), 100, current_video_buffer, 1);
	else if (status == STATUS_SO_SET_SC_CLEAR)
		put_string("Status: SO=1  SC=0", GET_X_CENTER(18), 100, current_video_buffer, 1);
	else if (status == STATUS_SO_CLEAR_SC_SET)
		put_string("Status: SO=0  SC=1", GET_X_CENTER(18), 100, current_video_buffer, 1);
	else if (status == STATUS_SO_SET_SC_SET)
		put_string("Status: SO=1  SC=1", GET_X_CENTER(18), 100, current_video_buffer, 1);
	wait_for_vsync();
	flip_and_show();
}


void clearSOandClearSC() {
	REG_R = 0x80F0;
	status = STATUS_SO_CLEAR_SC_CLEAR;
}


void setSOandClearSC() {
	REG_R = 0x80F8;
	status = STATUS_SO_SET_SC_CLEAR;
}


void clearSOandSetSC() {
	REG_R = 0x80F1;
	status = STATUS_SO_CLEAR_SC_SET;
}


void setSOandSetSC() {
	REG_R = 0x80FF;
	status = STATUS_SO_SET_SC_SET;
}


int AgbMain(void) {
	int i;

	REG_DISPCNT = MODE4 | BG2_ENABLE;
	for (i = 0; i < 256; i++)
		BG_PALETTE_MEM[i] = fontPalette[i];
	while (1) {
		show_main_screen();
		if (PRESSED(BUTTON_START))
			clearSOandClearSC();
		else if (PRESSED(BUTTON_A))
			setSOandClearSC();
		else if (PRESSED(BUTTON_B))
			clearSOandSetSC();
		else if (PRESSED(BUTTON_SELECT))
			setSOandSetSC();
	}
}

